
package dlog;

import java.net.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Server listening for data log requests.
 * 
 * @author Johan Hagelbäck (johan.hagelback@lnu.se)
 */
public class DLServer 
{
    // Server port
    private int port;
    // Data file logger
    private FileLogger log;
    
    /**
     * Creates a server at the specified port.
     * 
     * @param port Server port
     * @param filename Logger file filename
     */
    public DLServer(int port, String filename)
    {
        this.port = port;
        log = new FileLogger(filename);
    }
    
    /**
     * Starts the server. The server runs until the application quits.
     */
    public void start()
    {
        try
        {
            // Start server
            ServerSocket ss = new ServerSocket(port);
            System.out.println("Server started at port " + port);
            while (true)
            {
                // Request received
                Socket s = ss.accept();
                
                // Read contents
                BufferedReader in = new BufferedReader(new InputStreamReader(s.getInputStream()));
                String line = in.readLine();
                while (line != null && line.length() != 0)
                {
                    if (line.startsWith("GET"))
                    {
                        parseLine(line);
                    }
                    line = in.readLine();
                }
                
                // Send back response, using the HTTP 1.1 protocol
                PrintWriter out = new PrintWriter(s.getOutputStream());
                out.print("HTTP/1.1 200 \r\n");
                out.print("Content-Type: text/plain\r\n");
                out.print("Connection: close\r\n");
                out.print("\r\n");
                out.print("OK \r\n");
                out.print("\r\n");
                
                // Close everything
                out.close();
                in.close();
                s.close();
            }
        }
        catch (Exception ex)
        {
            System.err.println("Server error: " + ex.getMessage());
            System.exit(1);
        }
    }
    
    /**
     * Parses a GET line to see if a data value is received.
     * 
     * @param line GET line
     */
    private void parseLine(String line)
    {
        try
        {
            // Split line
            String[] parts = line.split(" ");
            
            // Error check
            if (!parts[0].equals("GET")) return;
            
            // Search for the log?value= part
            for (String t : parts)
            {
                if (t.contains("log?value="))
                {
                    String[] req = t.split("=");
                    // Second token is the value
                    double value = Double.parseDouble(req[1]);
                    // Get current timestamp
                    String tstmp = getCurrentTimeStamp();
                    // Output and log received data
                    System.out.println("  [" + tstmp + FileLogger.separator + value + "]");
                    log.log(tstmp, value);
                }
            }
        }
        catch (Exception ex)
        {
            System.err.println("Invalid request line: " + line);
        }
    }
    
    /**
     * Formats the current timestamo to a string.
     * 
     * @return Current timestamp string
     */
    private String getCurrentTimeStamp() 
    {
        SimpleDateFormat dfmt = new SimpleDateFormat("yyyy-MM-dd;HH:mm:ss");
        Date now = new Date();
        String dstr = dfmt.format(now);
        return dstr;
    }
}
